

# OMEGA_SET_PREFIX(PREFIX)
#---------------------------
AC_DEFUN([OMEGA_SET_PREFIX],
[
    unset CDPATH
    # make $PIPE_HOME the default for the installation
    AC_PREFIX_DEFAULT($1)

    if test "x$prefix" = "xNONE"; then
        prefix=$ac_default_prefix
        ac_configure_args="$ac_configure_args --prefix $prefix"
    fi

    if test "x$exec_prefix" = "xNONE"; then
        exec_prefix=$prefix
    fi

])


# OMEGA_SET_VERSION_INFO(VERSION, [CURRENT], [REVISION], [AGE])
#----------------------------------------------------------------
# Setup various version information, especially the libtool versioning
AC_DEFUN([OMEGA_SET_VERSION_INFO],
[
    omega_version=`echo "$1" | sed -e 's/[[a-z,A-Z]].*$//'`

    omega_major_version=`echo "$omega_version" | \
        sed 's/\([[0-9]]*\).\(.*\)/\1/'`
    omega_minor_version=`echo "$omega_version" | \
        sed 's/\([[0-9]]*\).\([[0-9]]*\)\(.*\)/\2/'`
    omega_micro_version=`echo "$omega_version" | \
        sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    if test -z "$omega_major_version"; then omega_major_version=0
    fi

    if test -z "$omega_minor_version"; then omega_minor_version=0
    fi

    if test -z "$omega_micro_version"; then omega_micro_version=0
    fi

    OMEGA_VERSION="$omega_version"
    OMEGA_MAJOR_VERSION=$omega_major_version
    OMEGA_MINOR_VERSION=$omega_minor_version
    OMEGA_MICRO_VERSION=$omega_micro_version

    if test -z "$4"; then OMEGA_INTERFACE_AGE=0
    else OMEGA_INTERFACE_AGE="$4"
    fi

    OMEGA_BINARY_AGE=`expr 100 '*' $OMEGA_MINOR_VERSION + $OMEGA_MICRO_VERSION`
    OMEGA_BINARY_VERSION=`expr 10000 '*' $OMEGA_MAJOR_VERSION + \
                          $OMEGA_BINARY_AGE`

    AC_SUBST(OMEGA_VERSION)
    AC_SUBST(OMEGA_MAJOR_VERSION)
    AC_SUBST(OMEGA_MINOR_VERSION)
    AC_SUBST(OMEGA_MICRO_VERSION)
    AC_SUBST(OMEGA_INTERFACE_AGE)
    AC_SUBST(OMEGA_BINARY_VERSION)
    AC_SUBST(OMEGA_BINARY_AGE)

    AC_DEFINE_UNQUOTED(OMEGA_MAJOR_VERSION, $OMEGA_MAJOR_VERSION,
                       [OMEGA major version number])
    AC_DEFINE_UNQUOTED(OMEGA_MINOR_VERSION, $OMEGA_MINOR_VERSION,
                       [OMEGA minor version number])
    AC_DEFINE_UNQUOTED(OMEGA_MICRO_VERSION, $OMEGA_MICRO_VERSION,
                       [OMEGA micro version number])
    AC_DEFINE_UNQUOTED(OMEGA_INTERFACE_AGE, $OMEGA_INTERFACE_AGE,
                       [OMEGA interface age])
    AC_DEFINE_UNQUOTED(OMEGA_BINARY_VERSION, $OMEGA_BINARY_VERSION,
                       [OMEGA binary version number])
    AC_DEFINE_UNQUOTED(OMEGA_BINARY_AGE, $OMEGA_BINARY_AGE,
                       [OMEGA binary age])

    ESO_SET_LIBRARY_VERSION([$2], [$3], [$4])
])


# OMEGA_SET_PATHS
#------------------
# Define auxiliary directories of the installed directory tree.
AC_DEFUN([OMEGA_SET_PATHS],
[

    if test -z "$plugindir"; then
        plugindir='${libdir}/esopipes-plugins/${PACKAGE}-${VERSION}'
    fi
    if test -z "$privatelibdir"; then
        privatelibdir='${libdir}/${PACKAGE}-${VERSION}'
    fi

    if test -z "$pipedocsdir"; then
        pipedocsdir='${datadir}/doc/esopipes/${PACKAGE}-${VERSION}/'
    fi
    htmldir='${pipedocsdir}/html'

    if test -z "$configdir"; then
#       configdir='${exec_prefix}/config'
        configdir='${datadir}/esopipes/${PACKAGE}-${VERSION}/config'
    fi


    if test "x$exec_prefix" != "xNONE"; then
        sex_exec_prefix=$exec_prefix/lib/${PACKAGE}-${VERSION}
        ac_configure_args="$ac_configure_args --exec_prefix $sex_exec_prefix"
    elif test "x$prefix" != "xNONE"; then
        sex_exec_prefix=$prefix/lib/${PACKAGE}-${VERSION}
        ac_configure_args="$ac_configure_args --exec_prefix $sex_exec_prefix"
    else
        sex_exec_prefix=$ac_default_prefix/lib/${PACKAGE}-${VERSION}
        ac_configure_args="$ac_configure_args --exec_prefix $sex_exec_prefix"
    fi


    AC_SUBST(plugindir)
    AC_SUBST(privatelibdir)
    AC_SUBST(pipedocsdir)
    AC_SUBST(htmldir)
    AC_SUBST(configdir)
    AC_SUBST(sex_exec_prefix)


    # Define a preprocesor symbol for the plugin search paths

    AC_DEFINE_UNQUOTED(OMEGA_PLUGIN_DIR, "esopipes-plugins",
                       [Plugin directory tree prefix])

    eval plugin_dir="$plugindir"
    plugin_path=`eval echo $plugin_dir | \
                sed -e "s/\/${PACKAGE}-${VERSION}.*$//"`

    AC_DEFINE_UNQUOTED(OMEGA_PLUGIN_PATH, "$plugin_path",
                       [Absolute path to the plugin directory tree])

    # Define the preprocessor symbols for the external executables
    # and the configuration files.



     eval d1="$configdir"
     eval d="$d1"
     eval omega_configdir="$d"

    AC_DEFINE_UNQUOTED(OMEGA_CONFIG_PATH, "$omega_configdir",
                       [Absolute path to the external program configuration files])


     eval d1="$sex_exec_prefix/bin"
     eval d="$d1"
     eval omega_bindir="$d"


    AC_DEFINE_UNQUOTED(OMEGA_BIN_PATH, "$omega_bindir",
                       [Absolute path to the external executables])

#    AC_DEFINE_UNQUOTED(OMEGA_BIN_PATH, "$omega_bindir",
#                       [Absolute path to the external executables])

])


# OMEGA_CREATE_SYMBOLS
#-----------------------
# Define include and library related makefile symbols
AC_DEFUN([OMEGA_CREATE_SYMBOLS],
[

    # Symbols for package include file and library search paths

    OMEGA_INCLUDES='-I$(top_srcdir)/omega -I$(top_builddir)/omega'
    OMEGA_LDFLAGS='-L$(top_builddir)/omega'

#    LIBFFTW_INCLUDES='-I$(top_builddir)/external/fftw/api'
#    LIBFFTW_LDFLAGS='-L$(top_builddir)/external/fftw'


    # Library aliases

    LIBOMEGA='$(top_builddir)/omega/libomega.la'
#    LIBFFTW='$(top_buiilddir)/external/fftw/libfftw3.la'
#	LIBIRPLIB='$(top_builddir)/irplib/libirplib.la'

    # Substitute the defined symbols

    AC_SUBST(OMEGA_INCLUDES)
    AC_SUBST(OMEGA_LDFLAGS)

#    AC_SUBST(LIBFFTW_INCLUDES)
#    AC_SUBST(LIBFFTW_LDFLAGS)

    AC_SUBST(LIBOMEGA)
#    AC_SUBST(LIBFFTW)
#	AC_SUBST(IRPLIB)

    # Check for CPL and user defined libraries
    AC_REQUIRE([CPL_CHECK_LIBS])
    AC_REQUIRE([ESO_CHECK_EXTRA_LIBS])

    all_includes='$(OMEGA_INCLUDES) $(CPL_INCLUDES) $(EXTRA_INCLUDES)'
    all_ldflags='$(OMEGA_LDFLAGS) $(CPL_LDFLAGS) $(EXTRA_LDFLAGS)'

    AC_SUBST(all_includes)
    AC_SUBST(all_ldflags)
])
