/* $Id: omega_dfs.c,v 1.2 2011-05-18 08:00:49 agabasch Exp $
 *
 * This file is part of the oc Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2011-05-18 08:00:49 $
 * $Revision: 1.2 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
   								Includes
 -----------------------------------------------------------------------------*/

#include <string.h>
#include <math.h>

#include "omega_dfs.h"


/**@{*/

/*----------------------------------------------------------------------------*/
/**
 * @defgroup oc_dfs  DFS related functions
 *
 * TBD
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Set the group as RAW or CALIB in a frameset
  @param    set     the input frameset
  @return   0 if ok, -1 in error case
 */
/*----------------------------------------------------------------------------*/
int oc_dfs_set_groups(cpl_frameset * set)
{
    cpl_frame   *   cur_frame ;
    const char  *   tag ;
    int             nframes ;
    int             i ;

    /* Check entries */
    if (set == NULL) return -1 ;

    /* Initialize */
    nframes = cpl_frameset_get_size(set) ;

    /* Loop on frames */
    for (i=0 ; i<nframes ; i++) {
        cur_frame = cpl_frameset_get_frame(set, i) ;
        tag = cpl_frame_get_tag(cur_frame) ;

        /* RAW frames */
        if (!strcmp(tag, DARK_RAW) ||
			!strcmp(tag, FRINGES_RAW) ||
			!strcmp(tag, GAIN_FLAT) ||
			!strcmp(tag, GAIN_DARK) ||
			!strcmp(tag, MBIAS_RAW) ||
			!strcmp(tag, LIFETEST_RAW) ||
			!strcmp(tag, MDOME_RAW) ||
			!strcmp(tag, DITHER_RAW) ||
			!strcmp(tag, OFFSET_RAW) ||
			!strcmp(tag, MTWIL_RAW) ||
			!strcmp(tag, RNOISE_RAW) ||
			!strcmp(tag, STARE_RAW) ||
			!strcmp(tag, JITTER_RAW) ||
			!strcmp(tag, SHUTTER_RAW) ||
			!strcmp(tag, STD_RAW)) {
            cpl_frame_set_group(cur_frame, CPL_FRAME_GROUP_RAW) ;
		}
        /* CALIB frames */
        else if (!strcmp(tag,OMEGA_CALIB_BIAS) ||
				!strcmp(tag, OMEGA_CALIB_BPM) ||
				!strcmp(tag, OMEGA_CALIB_CPM ) ||
				!strcmp(tag, OMEGA_CALIB_DARK) ||
				!strcmp(tag, OMEGA_CALIB_DOME) ||
				!strcmp(tag, OMEGA_CALIB_EXTCUR) ||
				!strcmp(tag, OMEGA_CALIB_FLAT) ||
				!strcmp(tag, OMEGA_CALIB_FRINGE) ||
				!strcmp(tag, OMEGA_CALIB_HPM) ||
				!strcmp(tag, OMEGA_CALIB_ILLFIT) ||
				!strcmp(tag, OMEGA_CALIB_ILLUM) ||
				!strcmp(tag, OMEGA_CALIB_MONIT) ||
				!strcmp(tag, OMEGA_CALIB_NSKY) ||
				!strcmp(tag, OMEGA_CALIB_RDNOISE) ||
				!strcmp(tag, OMEGA_CALIB_REFSTAR) ||
				!strcmp(tag, OMEGA_CALIB_TWIL) ||
				!strcmp(tag, OMEGA_CALIB_COLTERMS) ||
				!strcmp(tag, OMEGA_USNOA2) ||
				!strcmp(tag, OMEGA_CALIB_EXTCUR) ||
				!strcmp(tag, REFMFLAT) ||
				!strcmp(tag, OMEGA_CALIB_ZP)) {
            cpl_frame_set_group(cur_frame, CPL_FRAME_GROUP_CALIB);
		}

    }

    return 0 ;
}

/**@}*/
