/* $Id: omega_stare.c,v 1.12 2012-03-29 12:17:12 agabasch Exp $
 *
 * This file is part of the OMEGA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2012-03-29 12:17:12 $
 * $Revision: 1.12 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "omega_wcscor.h"
#include "omega_background.h"
#include "omega_bpm.h"
#include "omega_cosmic.h"
#include "omega_dfs.h"
#include "omega_flats.h"
#include "omega_fits.h"
#include "omega_pfits.h"
#include "omega_satellites.h"
#include "omega_science.h"
#include "omega_stats.h"
#include "omega_trim.h"
#include "omega_utils.h"

#define RECIPE "omega_science"

/**
 * @defgroup science_tools   Science Tools
 *
 * @par Synopsis
 *   Functions to process science frames
 *
 *   @code
 *   #include "omega_wcscor.h"
 *   #include "omega_background.h"
 *   #include "omega_bpm.h"
 *   #include "omega_cosmic.h"
 *   #include "omega_dfs.h"
 *   #include "omega_flats.h"
 *   #include "omega_fits.h"
 *   #include "omega_pfits.h"
 *   #include "omega_satellites.h"
 *   #include "omega_science.h"
 *   #include "omega_stats.h"
 *   #include "omega_trim.h"
 *   #include "omega_utils.h"
 *   @endcode
 *
 */
/**@{*/

/**
 * @brief	Reduces a science image (subtract bias, divide by the flat,
 * create weight/contribution map, calculates ZP and WCS information.
 *
 * @param	pars	Parameters list
 * @param	nraw	Number of raw science frames
 * @param	ext		FITS extension number
 *
 * @return	0 in success and -1 in case of error.
 *
 */
/*int omega_science_process(cpl_parameterlist *pars, int nraw, int ext)
{

	int i = 0;
	double exptime = 0.0;
	const char *tmpreduced = "omega_stare_reduced_temp.fits";
	const char *bckname = "omega_stare_back_temp.fits";
	const char *weight_name = "omega_stare_weight_temp.fits";
	const char *starename = "omega_stare_temp.fits";
	const char *smoothed = "omega_stare_smoothed.fits";
	const char *refname,*monit_name, *extname;
	const char *colname = NULL;

	cpl_image *bpm_img, *mask_img, *image, *weight, *weight_img;
	cpl_image *illum_img = NULL;
	cpl_image *backimg = NULL;
	cpl_image *reduced = NULL;
	cpl_image *smooth = NULL;
	cpl_mask *satu_map,*bpm_map;
	cpl_mask *cosmic_map, *satellite_map;
	cpl_propertylist *plist;
	cpl_table *phototbl, *exttbl;
	const cpl_frame *stareframe;

	 All member of ps structure are deallocated in main recipe

	 Load master flat
	ps.mflat = cpl_image_load(cpl_frame_get_filename(ps.mflatfr),CPL_TYPE_FLOAT,0,ext);
	if (ps.mflat == NULL){
		cpl_msg_error(cpl_func,"Cannot load Master Flat image",cpl_error_get_message());
		return -1;
	}
	if(cpl_image_get_mean(ps.mflat) < 0.1){
		cpl_msg_error(cpl_func,"Master Flat image is NULL");
		return -1;
	}

	 Load bad pixels map
	bpm_img = cpl_image_load(cpl_frame_get_filename(ps.bpmfr), CPL_TYPE_INT, 0, ext);
	if(bpm_img == NULL){
		cpl_msg_error(cpl_func, "Cannot load BPM", cpl_error_get_message());
		return -1;
	}

	 Load optional illumination map
	if(ps.illumfr != NULL)
		illum_img = cpl_image_load(cpl_frame_get_filename(ps.illumfr),CPL_TYPE_FLOAT,0,ext);

	 Loop through all science frames in frame set
	stareframe = cpl_frameset_get_first_const(ps.scilist);

	for (i = 0; i < nraw; i++){
		cpl_msg_info(cpl_func,"Reducing image %s", cpl_frame_get_filename(stareframe));

		Create a Saturated Pixels Map
		satu_map = create_saturated_map(stareframe, omega_science_config.oc, ext, pars);
		if (satu_map == NULL)
			cpl_msg_warning(cpl_func,"Null saturation map");

		Create logical mask from saturated map and input bad pixels map
		bpm_map = cpl_mask_threshold_image_create(bpm_img, 0.5, 1.5) ;
		if(satu_map != NULL){
			cpl_mask_and(bpm_map, satu_map);
			freemask(satu_map);
		}
		mask_img = cpl_image_new_from_mask(bpm_map);
		freemask(bpm_map);

		Create Reduced image
		reduced = omega_reduce_science(stareframe, mask_img, pars, ext);
		if(reduced == NULL){
			freeimage(ps.mflat);
			ps.mflat = NULL;
			freeimage(mask_img);
			freeimage(bpm_img);
			freeimage(illum_img);
			cpl_msg_error(cpl_func,"Cannot reduce image");
			return -1;
		}

		Create Cosmic Rays Map
		image = cpl_image_cast(mask_img,CPL_TYPE_FLOAT);
		weight = cpl_image_multiply_create(ps.mflat,image);
		freeimage(image);

		if(ps.illumfr != NULL)
			cpl_image_divide(weight,illum_img);

		cpl_image_power(weight, 0.5);
		cpl_image_save(reduced, tmpreduced, BITPIX, NULL, CPL_IO_DEFAULT);
		omega_create_background(pars, tmpreduced, bckname);
		backimg = cpl_image_load(bckname,CPL_TYPE_FLOAT, 0, 0);
		image = cpl_image_subtract_create(reduced,backimg);
		cpl_image_multiply(image,weight);
		freeimage(backimg);
		freeimage(weight);

		cosmic_map = detcosmic(image, pars);
		if(cosmic_map == NULL)
			cpl_msg_warning(cpl_func,"Cannot create cosmic rays map");

		Create Satellite Tracks Map
		FIXME: revise this function
		satellite_map = detsat(image, pars);
		if(satellite_map == NULL)
			cpl_msg_warning(cpl_func,"Cannot create satellite tracks map");

		freeimage(image);

		 Create Weight Map
		weight_img = create_weightframe(ps.mflat, mask_img, cosmic_map, satellite_map);
		if(weight_img == NULL){
			cpl_msg_warning(cpl_func,"Cannot create weight map");
			weight_img = NULL;
		}

		if((illum_img != NULL) && (weight_img != NULL)) {
			cpl_image_divide(weight_img, illum_img);
		}

		freeimage(mask_img);
		freemask(cosmic_map);
		freemask(satellite_map);

		 Multiply each science by its weight frame
		//     	cpl_image_save(weight_img, weight_name, BITPIX, NULL, CPL_IO_DEFAULT);
		cpl_image_multiply(reduced, weight_img);

		if(i == 0){
			ps.reduced = cpl_image_duplicate(reduced);
			ps.weight = cpl_image_duplicate(weight_img);
		}
		else{
			cpl_image_add(ps.reduced, reduced);
			cpl_image_add(ps.weight, weight_img);
		}
		freeimage(reduced);
		freeimage(weight_img);

		 Get next frame
		stareframe = cpl_frameset_get_next_const(ps.scilist);
	}

	freeimage(ps.mflat);
	ps.mflat = NULL;
	freeimage(bpm_img);
	freeimage(illum_img);

	 Weighted average of STARE frames
	if(cpl_image_divide(ps.reduced, ps.weight) != CPL_ERROR_NONE){
		cpl_msg_error(cpl_func,"Cannot create weighted average of images. %s", cpl_error_get_message());
		//    	freeimage(sumwght);
		dummy = 1;
		return 1;
	}

	 Cast weight map to INT
	ps.weight = cpl_image_cast(ps.weight,CPL_TYPE_INT);

	 Correct coordinates after trimming
	ps.astrolist = cpl_propertylist_duplicate(ps.eh);
	if(omega_shift_refpix(ps.firstframe, ext, ps.astrolist) != 0){
		cpl_msg_warning(cpl_func,"Cannot shift reference pixel of trimmed image");
		freeplist(ps.astrolist);
		ps.astrolist = NULL;
		dummy = 1;
		return 1;
	}


	 * Calculate new zeropoint using photometric table

	photom_tbl = cpl_table_load(cpl_frame_get_filename(ps.photomfr),ext,0);
	if(photom_tbl == NULL){
		cpl_msg_error(cpl_func,"Cannot load photometric table. %s",cpl_error_get_message());
		return -1;
	}

	zeropoint = cpl_table_get_float(photom_tbl,"ZEROPOINT",0,NULL);
	extinction = cpl_table_get_float(photom_tbl,"EXTINCTION",0,NULL);
	omega_sci_qc.ZP = zeropoint + 2.5* log10 (omega_pfits_get_exptime(ps.eh));
	omega_sci_qc.ZP -= extinction*omega_pfits_get_mean_airmass(ps.ph,&airmass);
	freetable(photom_tbl);

	 Add newZP to header of product
	cpl_propertylist_update_float(ps.astrolist,"ESO QC SCI ZEROPNT",omega_sci_qc.ZP);
	cpl_propertylist_set_comment(ps.astrolist,"ESO QC SCI ZEROPNT", "new zeropoint");


	 * Use pattern matching to detect stars.
	 * Smooth image before creating catalogue

	smooth = omega_smooth_image(ps.reduced, 7);

	 Save star as intermediate product
	cpl_propertylist_save(NULL,smoothed, CPL_IO_CREATE);
	cpl_image_save(smooth, smoothed, BITPIX, ps.astrolist, CPL_IO_EXTEND);
	freeimage(smooth);


	 * Calculate WCS distortion for image

	plist = omega_match_points(smoothed, ps.usnoa2, pars,
			&omega_sci_qc.nmatches, &omega_sci_qc.mratio);

	if(plist == NULL){
		cpl_msg_warning(cpl_func,"Error in astrometric calculation");
		freeplist(plist);
		dummy = 0;
		return 1;
	}

	 Append WCS header to extension header
	if(omega_pfits_update_header(&ps.astrolist, plist) != 0)
		cpl_msg_debug(cpl_func,"Cannot append WCS header to extension header");

	freeplist(plist);


	 * NOTE: Need a secondary standard star catalogue to
	 * calculate ZP for all science frames!!!


	return 0;
}*/


/**
 * @brief	Reduces a science image (subtract bias, divide by the flat,
 * 			create weight/contribution map, calculates ZP and WCS information.
 *
 * @param scifits	omega_fits structure of science image
 * @param mbias		master bias
 * @param mflat		master flat
 * @param bpm		bad pixels image
 * @param photom	photometric table
 * @param usnoa2	master USNOA2 table
 * @param illum		Optional illumination correction image
 * @param fringes	Optional fringes image
 * @param nsky		Optional night sky flat
 * @param pars		Parameters list
 * @param ext		FITS extension to load
 *
 * @return	sfits	omega_fits structure of reduced image or NULL
 * 					in case of error.
 */
omega_fits *omega_jitter_process(omega_fits *scifits,
		                       const cpl_image *mbias,
		                       const cpl_image *mflat,
						       const cpl_image *bpm,
						       const cpl_table *photom,
						       const cpl_frame *usnoa2,
						       const cpl_image *illum,
						       const cpl_image *fringes,
						       const cpl_image *nsky,
						       double * zeropoint_final,
						       cpl_parameterlist *pars, int ext)
{

	int i = 0;
	int oc = 0;
	int sub_back = 0;
	int nmatches = 0;
	double ZP = 0.0;
	double zeropoint = 0.0;
	double zeropoint_error = 0.0;
	double extinction = 0.0;
	double median = 1.0;
	double mratio = 0.0;
	double exptime = 0.0;
	double airmass = 0.0;
	double gain = 1.0;
	const char *smoothed = "omega_sci_smoothed.fits";
	const char *smoothed_weight = "omega_sci_smoothed_weight.fits";

	cpl_image *mask_img, *image, *trim, *reduced, *combmap;
	cpl_image *back = NULL;
	cpl_image *smooth = NULL;
	cpl_mask *satu_map,*bpm_map, *mask;
	cpl_mask *cosmic, *satellite;
	cpl_propertylist *plist, *astrolist, *wcslist;
	cpl_parameter *p;
	omega_fits *simplefits;
	double mag_zpt_no_std_table=0.0;
	double mag_zpt_err_no_std_table=0.0;
	double ext_no_std_table=0.0;

	if((scifits == NULL) || (mbias == NULL) || (mflat == NULL) ||
	   (bpm == NULL) || (usnoa2 == NULL) || (pars == NULL))
		return NULL;


/*	if(cpl_image_get_mean(mflat) < 0.1){
		cpl_msg_error(cpl_func,"Master Flat image is NULL");
		return NULL;
	}*/

	/* Get a few parameters */
	p = cpl_parameterlist_find(pars, "omega.omega_science.OverscanMethod") ;
	oc = cpl_parameter_get_int(p);
	p = cpl_parameterlist_find(pars,"omega.omega_science.SubtractBackground");
	sub_back = cpl_parameter_get_bool(p);


	p = cpl_parameterlist_find(pars, "omega.omega_science.mag_zpt_no_std_table") ;
	mag_zpt_no_std_table = cpl_parameter_get_double(p);
	p = cpl_parameterlist_find(pars,"omega.omega_science.mag_zpt_err_no_std_table");
	mag_zpt_err_no_std_table = cpl_parameter_get_double(p);
	p = cpl_parameterlist_find(pars,"omega.omega_science.ext_no_std_table");
	ext_no_std_table = cpl_parameter_get_double(p);


	/* Calculate new zeropoint using photometric table or use default values*/

	if(photom != NULL){
		zeropoint       = cpl_table_get_double(photom,"ZEROPOINT",0,NULL);
		zeropoint_error = cpl_table_get_double(photom, "ZEROPOINT_ERR", 0, NULL);
		extinction      = cpl_table_get_double(photom,"EXTINCTION",0,NULL);
	}
	else {
		zeropoint       = mag_zpt_no_std_table;
		zeropoint_error = mag_zpt_err_no_std_table;
		extinction      = ext_no_std_table;
	}
	/* Trim and overscan correct all images */
	if((trim = omega_trim_oscan_correct(scifits, oc)) == NULL){
		cpl_msg_error(cpl_func,"Unable to trim image");
		return NULL;
	}

	/* Correct coordinates after trimming */
	astrolist = omega_fits_get_ehu(scifits);
	omega_shift_refpix(omega_fits_get_frame(scifits), ext, astrolist);

	/* Add newZP to header of product */
	plist = omega_fits_get_phu(scifits);
	if((exptime = omega_pfits_get_exptime(astrolist)) <= 0.0)
		exptime = omega_pfits_get_exptime(plist);


	ZP = zeropoint + 2.5* log10(exptime);
	omega_pfits_get_mean_airmass(plist,&airmass);
	ZP -= extinction*airmass;

	cpl_propertylist_update_double(astrolist,"ESO QC SCI ZEROPNT",ZP);
	cpl_propertylist_set_comment(astrolist,"ESO QC SCI ZEROPNT", "new zp with flux in ADU");
	cpl_propertylist_update_double(astrolist, "ESO QC SCI ZEROPNT ERR", zeropoint_error) ;
	cpl_propertylist_set_comment(astrolist, "ESO QC SCI ZEROPNT ERR", "zp error");


	/*Get gain*/
	omega_pfits_get_conad(astrolist,&gain);

	cpl_propertylist_update_double(astrolist,"ESO QC SCI ZEROPNT_ELECTRON",ZP+(2.5*log10(gain)));
	cpl_propertylist_set_comment(astrolist,"ESO QC SCI ZEROPNT_ELECTRON", "new zp with flux in e-");
	cpl_propertylist_update_double(astrolist, "ESO QC SCI ZEROPNT_ELECTRON ERR", zeropoint_error) ;
	cpl_propertylist_set_comment(astrolist, "ESO QC SCI ZEROPNT_ELECTRON ERR", "zp error");

	*zeropoint_final=ZP;

	/* Subtract the bias */
	omega_biascor(trim, mbias);

	/* Divide by the flat */
	if(omega_flatcor(trim, mflat, nsky) != 0){
		cpl_msg_error(cpl_func,"Error in flat correction");
		freeimage(trim);
		plist = NULL;
		astrolist = NULL;
		return NULL;
	}

	/* Create BPM+SATURATION mask */
	satu_map = omega_saturated_map(trim, oc, pars);

	bpm_map = cpl_mask_threshold_image_create(bpm, 0.5, 1.5) ;
	if(satu_map != NULL){
//		cpl_mask_and(bpm_map, satu_map);
		cpl_mask_or(bpm_map, satu_map);
		freemask(satu_map);
	}
	mask_img = cpl_image_new_from_mask(bpm_map);
    /*Add the bad pixel mask to the image*/
    cpl_image_reject_from_mask(trim, bpm_map);
	freemask(bpm_map);

	/* Correct for fringing */
	if(fringes != NULL){
		omega_fringecor(trim, fringes, mask_img, pars);
	}

	/* Get background */
	back = omega_background(trim, pars);

	if(back != NULL){
		median = cpl_image_get_median(back);
		cpl_propertylist_update_double(astrolist, "ESO DRS MEDIAN BKG", median);
	}
	if(back == NULL)
		cpl_msg_debug(cpl_func,"Unable to create background image");

	/* Detect cosmic rays */
	image = cpl_image_subtract_create(trim, back);
	cosmic = omega_cosmic_rays(image,mflat,illum,mask_img,pars);

	/* Subtract background from science */
	if(sub_back == 1){
		freeimage(trim);
		trim = cpl_image_duplicate(image);
	}

	freeimage(image);
	freeimage(back);

	/* Detect satellite tracks in background subtracted image */
	satellite = detsat(trim, pars);

	/* Create combination map (BPM,saturated,cosmic,satellite) */
	/* The bad pixels are set to CPL_BINARY_0 in combmap */
	combmap = create_weightframe(mflat,mask_img,cosmic,satellite);
//	cpl_image_save(combmap,"combmap.fits",CPL_BPP_IEEE_FLOAT,NULL,CPL_IO_CREATE);

	freemask(satellite);
	freemask(cosmic);
	freeimage(mask_img);

	/* Save median value of image in header */
	mask = cpl_mask_threshold_image_create(combmap, 0.5, 1.5);
	cpl_mask_not(mask);
	cpl_image_reject_from_mask(trim, mask);
	median = cpl_image_get_median(trim);
	cpl_propertylist_update_double(astrolist, "ESO DRS MEDIAN", median);


	/* Clean bad pixels */
	if(cpl_detector_interpolate_rejected(trim) != CPL_ERROR_NONE)
		cpl_msg_debug(cpl_func,"Cannot clean rejected pixels. %s",cpl_error_get_message());

	/*The image should remember the bad pixels even after interpolation!*/


	/*
	 * Use pattern matching to detect stars.
	 */

	/* Smooth image before creating catalogue */
	smooth = omega_smooth_image(trim, 1);

	cpl_image_reject_from_mask(trim, mask);
	freemask(mask);

	/* Save star as intermediate product */
	cpl_propertylist_save(plist,smoothed, CPL_IO_CREATE);
	cpl_image_save(smooth, smoothed, CPL_BPP_IEEE_FLOAT, astrolist, CPL_IO_EXTEND);

	/* Save weight map  as intermediate product */
	cpl_propertylist_save(plist,smoothed_weight, CPL_IO_CREATE);
	cpl_image_save(combmap, smoothed_weight, CPL_BPP_IEEE_FLOAT, astrolist, CPL_IO_EXTEND);

	freeimage(smooth);
	freeimage(combmap);
	/*
	 * Calculate WCS distortion for simple image
	 */
	wcslist = omega_match_points(smoothed, smoothed_weight, usnoa2, pars,ext,&nmatches, &mratio);

	/* Debugging information */
	cpl_msg_debug(cpl_func,"Number matches between USNOA2 cat and stars. %d", nmatches);
	cpl_msg_debug(cpl_func,"ratio USNOA2 matches/detected stars. %g", mratio);

	if(wcslist == NULL){
		cpl_msg_warning(cpl_func,"Unable to calculate WCS distortion for image. %s",
							cpl_error_get_message());
	}

	/* Append WCS header to extension header */
	omega_pfits_update_header(astrolist, wcslist);
	freeplist(wcslist);

	/* Wrap product into omega fits structure using 1st frame headers */
	simplefits = omega_fits_wrap(trim,NULL,plist,astrolist);

//	plist = NULL;
//	astrolist = NULL;

	return simplefits;

}

/*
 * Copied from omega_reduce_std()
 */
/*
cpl_image *omega_reduce_science(const cpl_frame *sciframe, cpl_image *mask_img,
		cpl_parameterlist *pars, int ext)
{

	const char *cpl_func = "make_std_reduced";
	const char *bckname = "omega_background_temp.fits";
	const char *tempname1111 = "omega_reduced_temp.fits";
	cpl_image *trim_img, *mbias_img, *mflat_img, *nsky_img;
	cpl_image *back_img = NULL;
	cpl_image *illum_img = NULL;
	cpl_image *fringe_img = NULL;
	cpl_image *reduced = NULL;
	cpl_mask  *bpmtemp_map;


	 Trim and overscan correct image
	trim_img = TrimOscanCorrect(sciframe, omega_science_config.oc, ext);
	if (trim_img == NULL) {
		cpl_msg_error(cpl_func,"Cannot trim image %s",cpl_error_get_message());
		return NULL;
	}

	Subtract the bias
	mbias_img = cpl_image_load(cpl_frame_get_filename(ps.mbiasfr), CPL_TYPE_FLOAT,0,ext);
	if (mbias_img == NULL) {
		cpl_msg_error(cpl_func,"Cannot load master bias %s", cpl_error_get_message());
		freeimage(trim_img);
		return NULL;
	}
	cpl_image_subtract(trim_img, mbias_img);
	freeimage(mbias_img);


	Correct for flat field. If there is a night sky flat, use it.
	if(ps.nskyfr != NULL){
		nsky_img = cpl_image_load(cpl_frame_get_filename(ps.nskyfr), CPL_TYPE_FLOAT, 0, ext);
		if(nsky_img != NULL){
			mflat_img = cpl_image_multiply_create(ps.mflat, nsky_img);
			cpl_image_divide(trim_img, mflat_img);
			freeimage(nsky_img);
			freeimage(mflat_img);
		}
		else {
			cpl_image_divide(trim_img, ps.mflat);
		}
	}
	else {
		cpl_image_divide(trim_img, ps.mflat);
	}

	Correct for fringing. Check if filter really requires fringing correction
	FIXME: should this be a fatal error or a warning?
	if(ps.fringesfr != NULL){
		fringe_img = cpl_image_load(cpl_frame_get_filename(ps.fringesfr),CPL_TYPE_FLOAT,0,ext);
		if(fringe_img == NULL) {
			cpl_msg_warning(cpl_func,"Cannot load Master Fringe image");
			reduced = cpl_image_duplicate(trim_img);
		}
		else {
			bpmtemp_map = cpl_mask_threshold_image_create(mask_img, 0.5, 1.5);
			cpl_msg_info(cpl_func,"Applying fringing correction");
			reduced = omega_fringecor(trim_img,fringe_img, bpmtemp_map, pars);
			if(reduced == NULL) {
				cpl_msg_warning(cpl_func,"Cannot apply fringing correction");
				reduced = cpl_image_duplicate(trim_img);
			}
			freemask(bpmtemp_map);
			freeimage(fringe_img);
		}

	}
	else {
		reduced = cpl_image_duplicate(trim_img);
	}

	freeimage(trim_img);

	 FIXME: review this
	 Illumination Correction and Background subtraction
	//    iillum = (illum == NULL) ? 0 : 1;

	if((ps.illumfr != NULL) && (omega_science_config.bckg == 0)){

		//    if (!(iillum == 0 && bckg == 1)) {
		Save temporary image to create a background in Sextractor
		cpl_image_save(reduced, tempname1111, BITPIX, NULL, CPL_IO_DEFAULT);

		     cpl_msg_info("","Measuring statistics with background");

        stats = omega_iter_stat_opts(reduced_img, 5.0, 5);
        cpl_msg_info("","Mean   :%10.3f",cpl_stats_get_mean(stats));
        cpl_msg_info("","Median :%10.3f",cpl_stats_get_median(stats));
        cpl_msg_info("","Stdev  :%10.3f",cpl_stats_get_stdev(stats));

		cpl_msg_info(cpl_func,"Doing background subtraction");
		if(omega_create_background(pars, tempname1111, bckname) != 0){
			cpl_msg_error(cpl_func,"Cannot create background image");
			freeimage(reduced);
			return NULL;
		}

		back_img = cpl_image_load(bckname, CPL_TYPE_FLOAT, 0, 0);
		if(back_img == NULL) {
			cpl_msg_warning(cpl_func,"Cannot load background image %s", bckname);
			freeimage(reduced);
			return NULL;
		}

		cpl_image_subtract(reduced, back_img);
		                      cpl_stats_delete(stats);
	}


	//    if(iillum == 1) {
	if (ps.illumfr != NULL){
		illum_img = cpl_image_load(cpl_frame_get_filename(ps.illumfr),CPL_TYPE_FLOAT, 0, ext);
		if(illum_img != NULL) {
			cpl_msg_info(cpl_func,"Applying illumination correction");
			cpl_image_multiply(reduced, illum_img);
			freeimage(illum_img);
		}
		else{
			cpl_msg_warning(cpl_func,"Cannot load Illumination Correction image");
		}
		if(omega_std_config.bckg == 1) {
			cpl_msg_info("","Restoring background to image");
			cpl_image_add(reduced, back_img);
		}
	}


	if (back_img != NULL){
		freeimage(back_img);
	}

	return reduced;


}
*/


/**@}*/
